@@ -1,9 +1,4 @@
-# -*- coding: utf-8 -*-
 """ command line options, ini-file and conftest.py processing. """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import argparse
 import copy
 import inspect
@@ -12,9 +7,20 @@ import shlex
 import sys
 import types
 import warnings
+from functools import lru_cache
+from types import TracebackType
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import Sequence
+from typing import Set
+from typing import Tuple
+from typing import Union
 
+import attr
 import py
-import six
 from packaging.version import Version
 from pluggy import HookimplMarker
 from pluggy import HookspecMarker
@@ -22,21 +28,25 @@ from pluggy import PluginManager
 
 import _pytest._code
 import _pytest.assertion
+import _pytest.deprecated
 import _pytest.hookspec  # the extension point definitions
 from .exceptions import PrintHelp
 from .exceptions import UsageError
 from .findpaths import determine_setup
 from .findpaths import exists
-from _pytest import deprecated
 from _pytest._code import ExceptionInfo
 from _pytest._code import filter_traceback
 from _pytest.compat import importlib_metadata
-from _pytest.compat import lru_cache
-from _pytest.compat import safe_str
+from _pytest.compat import TYPE_CHECKING
 from _pytest.outcomes import fail
 from _pytest.outcomes import Skipped
+from _pytest.pathlib import Path
 from _pytest.warning_types import PytestConfigWarning
 
+if TYPE_CHECKING:
+    from typing import Type
+
+
 hookimpl = HookimplMarker("pytest")
 hookspec = HookspecMarker("pytest")
 
@@ -45,10 +55,10 @@ class ConftestImportFailure(Exception):
     def __init__(self, path, excinfo):
         Exception.__init__(self, path, excinfo)
         self.path = path
-        self.excinfo = excinfo
+        self.excinfo = excinfo  # type: Tuple[Type[Exception], Exception, TracebackType]
 
 
-def main(args=None, plugins=None):
+def main(args=None, plugins=None) -> "Union[int, _pytest.main.ExitCode]":
     """ return exit code, after performing an in-process test run.
 
     :arg args: list of command line arguments.
@@ -56,7 +66,7 @@ def main(args=None, plugins=None):
     :arg plugins: list of plugin objects to be auto-registered during
                   initialization.
     """
-    from _pytest.main import EXIT_USAGEERROR
+    from _pytest.main import ExitCode
 
     try:
         try:
@@ -73,23 +83,29 @@ def main(args=None, plugins=None):
                 if exc_info.traceback
                 else exc_info.exconly()
             )
-            formatted_tb = safe_str(exc_repr)
+            formatted_tb = str(exc_repr)
             for line in formatted_tb.splitlines():
                 tw.line(line.rstrip(), red=True)
-            return 4
+            return ExitCode.USAGE_ERROR
         else:
             try:
-                return config.hook.pytest_cmdline_main(config=config)
+                ret = config.hook.pytest_cmdline_main(
+                    config=config
+                )  # type: Union[ExitCode, int]
+                try:
+                    return ExitCode(ret)
+                except ValueError:
+                    return ret
             finally:
                 config._ensure_unconfigure()
     except UsageError as e:
         tw = py.io.TerminalWriter(sys.stderr)
         for msg in e.args:
             tw.line("ERROR: {}\n".format(msg), red=True)
-        return EXIT_USAGEERROR
+        return ExitCode.USAGE_ERROR
 
 
-class cmdline(object):  # compatibility namespace
+class cmdline:  # compatibility namespace
     main = staticmethod(main)
 
 
@@ -148,16 +164,22 @@ default_plugins = essential_plugins + (
     "warnings",
     "logging",
     "reports",
+    "faulthandler",
 )
 
 builtin_plugins = set(default_plugins)
 builtin_plugins.add("pytester")
 
 
-def get_config(args=None):
+def get_config(args=None, plugins=None):
     # subsequent calls to main will create a fresh instance
     pluginmanager = PytestPluginManager()
-    config = Config(pluginmanager)
+    config = Config(
+        pluginmanager,
+        invocation_params=Config.InvocationParams(
+            args=args or (), plugins=plugins, dir=Path().resolve()
+        ),
+    )
 
     if args is not None:
         # Handle any "-p no:plugin" args.
@@ -181,7 +203,6 @@ def get_plugin_manager():
 
 
 def _prepareconfig(args=None, plugins=None):
-    warning = None
     if args is None:
         args = sys.argv[1:]
     elif isinstance(args, py.path.local):
@@ -190,19 +211,15 @@ def _prepareconfig(args=None, plugins=None):
         msg = "`args` parameter expected to be a list or tuple of strings, got: {!r} (type: {})"
         raise TypeError(msg.format(args, type(args)))
 
-    config = get_config(args)
+    config = get_config(args, plugins)
     pluginmanager = config.pluginmanager
     try:
         if plugins:
             for plugin in plugins:
-                if isinstance(plugin, six.string_types):
+                if isinstance(plugin, str):
                     pluginmanager.consider_pluginarg(plugin)
                 else:
                     pluginmanager.register(plugin)
-        if warning:
-            from _pytest.warnings import _issue_warning_captured
-
-            _issue_warning_captured(warning, hook=config.hook, stacklevel=4)
         return pluginmanager.hook.pytest_cmdline_parse(
             pluginmanager=pluginmanager, args=args
         )
@@ -211,6 +228,19 @@ def _prepareconfig(args=None, plugins=None):
         raise
 
 
+def _fail_on_non_top_pytest_plugins(conftestpath, confcutdir):
+    msg = (
+        "Defining 'pytest_plugins' in a non-top-level conftest is no longer supported:\n"
+        "It affects the entire test suite instead of just below the conftest as expected.\n"
+        "  {}\n"
+        "Please move it to a top level conftest file at the rootdir:\n"
+        "  {}\n"
+        "For more information, visit:\n"
+        "  https://docs.pytest.org/en/latest/deprecations.html#pytest-plugins-in-non-top-level-conftest-files"
+    )
+    fail(msg.format(conftestpath, confcutdir), pytrace=False)
+
+
 class PytestPluginManager(PluginManager):
     """
     Overwrites :py:class:`pluggy.PluginManager <pluggy.PluginManager>` to add pytest-specific
@@ -222,15 +252,19 @@ class PytestPluginManager(PluginManager):
     """
 
     def __init__(self):
-        super(PytestPluginManager, self).__init__("pytest")
-        self._conftest_plugins = set()
+        super().__init__("pytest")
+        # The objects are module objects, only used generically.
+        self._conftest_plugins = set()  # type: Set[object]
 
         # state related to local conftest plugins
-        self._dirpath2confmods = {}
-        self._conftestpath2mod = {}
+        # Maps a py.path.local to a list of module objects.
+        self._dirpath2confmods = {}  # type: Dict[Any, List[object]]
+        # Maps a py.path.local to a module object.
+        self._conftestpath2mod = {}  # type: Dict[Any, object]
         self._confcutdir = None
         self._noconftest = False
-        self._duplicatepaths = set()
+        # Set of py.path.local's.
+        self._duplicatepaths = set()  # type: Set[Any]
 
         self.add_hookspecs(_pytest.hookspec)
         self.register(self)
@@ -249,16 +283,6 @@ class PytestPluginManager(PluginManager):
         # Used to know when we are importing conftests after the pytest_configure stage
         self._configured = False
 
-    def addhooks(self, module_or_class):
-        """
-        .. deprecated:: 2.8
-
-        Use :py:meth:`pluggy.PluginManager.add_hookspecs <PluginManager.add_hookspecs>`
-        instead.
-        """
-        warnings.warn(deprecated.PLUGIN_MANAGER_ADDHOOKS, stacklevel=2)
-        return self.add_hookspecs(module_or_class)
-
     def parse_hookimpl_opts(self, plugin, name):
         # pytest hooks are always prefixed with pytest_
         # so we avoid accessing possibly non-readable attributes
@@ -270,7 +294,7 @@ class PytestPluginManager(PluginManager):
             return
 
         method = getattr(plugin, name)
-        opts = super(PytestPluginManager, self).parse_hookimpl_opts(plugin, name)
+        opts = super().parse_hookimpl_opts(plugin, name)
 
         # consider only actual functions for hooks (#3775)
         if not inspect.isroutine(method):
@@ -289,9 +313,7 @@ class PytestPluginManager(PluginManager):
         return opts
 
     def parse_hookspec_opts(self, module_or_class, name):
-        opts = super(PytestPluginManager, self).parse_hookspec_opts(
-            module_or_class, name
-        )
+        opts = super().parse_hookspec_opts(module_or_class, name)
         if opts is None:
             method = getattr(module_or_class, name)
 
@@ -308,7 +330,7 @@ class PytestPluginManager(PluginManager):
         return opts
 
     def register(self, plugin, name=None):
-        if name in ["pytest_catchlog", "pytest_capturelog"]:
+        if name in _pytest.deprecated.DEPRECATED_EXTERNAL_PLUGINS:
             warnings.warn(
                 PytestConfigWarning(
                     "{} plugin has been merged into the core, "
@@ -318,7 +340,7 @@ class PytestPluginManager(PluginManager):
                 )
             )
             return
-        ret = super(PytestPluginManager, self).register(plugin, name)
+        ret = super().register(plugin, name)
         if ret:
             self.hook.pytest_plugin_registered.call_historic(
                 kwargs=dict(plugin=plugin, manager=self)
@@ -403,12 +425,6 @@ class PytestPluginManager(PluginManager):
         else:
             directory = path
 
-        if six.PY2:  # py2 is not using lru_cache.
-            try:
-                return self._dirpath2confmods[directory]
-            except KeyError:
-                pass
-
         # XXX these days we may rather want to use config.rootdir
         # and allow users to opt into looking into the rootdir parent
         # directories instead of requiring to specify confcutdir
@@ -418,10 +434,7 @@ class PytestPluginManager(PluginManager):
                 continue
             conftestpath = parent.join("conftest.py")
             if conftestpath.isfile():
-                # Use realpath to avoid loading the same conftest twice
-                # with build systems that create build directories containing
-                # symlinks to actual files.
-                mod = self._importconftest(conftestpath.realpath())
+                mod = self._importconftest(conftestpath)
                 clist.append(mod)
         self._dirpath2confmods[directory] = clist
         return clist
@@ -436,8 +449,14 @@ class PytestPluginManager(PluginManager):
         raise KeyError(name)
 
     def _importconftest(self, conftestpath):
+        # Use a resolved Path object as key to avoid loading the same conftest twice
+        # with build systems that create build directories containing
+        # symlinks to actual files.
+        # Using Path().resolve() is better than py.path.realpath because
+        # it resolves to the correct path/drive in case-insensitive file systems (#5792)
+        key = Path(str(conftestpath)).resolve()
         try:
-            return self._conftestpath2mod[conftestpath]
+            return self._conftestpath2mod[key]
         except KeyError:
             pkgpath = conftestpath.pypkgpath()
             if pkgpath is None:
@@ -449,21 +468,12 @@ class PytestPluginManager(PluginManager):
                     and self._configured
                     and not self._using_pyargs
                 ):
-                    from _pytest.deprecated import (
-                        PYTEST_PLUGINS_FROM_NON_TOP_LEVEL_CONFTEST,
-                    )
-
-                    fail(
-                        PYTEST_PLUGINS_FROM_NON_TOP_LEVEL_CONFTEST.format(
-                            conftestpath, self._confcutdir
-                        ),
-                        pytrace=False,
-                    )
+                    _fail_on_non_top_pytest_plugins(conftestpath, self._confcutdir)
             except Exception:
                 raise ConftestImportFailure(conftestpath, sys.exc_info())
 
             self._conftest_plugins.add(mod)
-            self._conftestpath2mod[conftestpath] = mod
+            self._conftestpath2mod[key] = mod
             dirpath = conftestpath.dirpath()
             if dirpath in self._dirpath2confmods:
                 for path, mods in self._dirpath2confmods.items():
@@ -485,7 +495,7 @@ class PytestPluginManager(PluginManager):
         while i < n:
             opt = args[i]
             i += 1
-            if isinstance(opt, six.string_types):
+            if isinstance(opt, str):
                 if opt == "-p":
                     try:
                         parg = args[i]
@@ -546,7 +556,7 @@ class PytestPluginManager(PluginManager):
         # "terminal" or "capture".  Those plugins are registered under their
         # basename for historic purposes but must be imported with the
         # _pytest prefix.
-        assert isinstance(modname, six.string_types), (
+        assert isinstance(modname, str), (
             "module name as text required, got %r" % modname
         )
         modname = str(modname)
@@ -564,20 +574,19 @@ class PytestPluginManager(PluginManager):
         try:
             __import__(importspec)
         except ImportError as e:
-            new_exc_message = 'Error importing plugin "%s": %s' % (
-                modname,
-                safe_str(e.args[0]),
+            new_exc_message = 'Error importing plugin "{}": {}'.format(
+                modname, str(e.args[0])
             )
             new_exc = ImportError(new_exc_message)
             tb = sys.exc_info()[2]
 
-            six.reraise(ImportError, new_exc, tb)
+            raise new_exc.with_traceback(tb)
 
         except Skipped as e:
             from _pytest.warnings import _issue_warning_captured
 
             _issue_warning_captured(
-                PytestConfigWarning("skipped plugin %r: %s" % (modname, e.msg)),
+                PytestConfigWarning("skipped plugin {!r}: {}".format(modname, e.msg)),
                 self.hook,
                 stacklevel=1,
             )
@@ -595,7 +604,7 @@ def _get_plugin_specs_as_list(specs):
     empty list is returned.
     """
     if specs is not None and not isinstance(specs, types.ModuleType):
-        if isinstance(specs, six.string_types):
+        if isinstance(specs, str):
             specs = specs.split(",") if specs else []
         if not isinstance(specs, (list, tuple)):
             raise UsageError(
@@ -613,7 +622,7 @@ def _ensure_removed_sysmodule(modname):
         pass
 
 
-class Notset(object):
+class Notset:
     def __repr__(self):
         return "<NOTSET>"
 
@@ -681,36 +690,80 @@ def _iter_rewritable_modules(package_files):
             if new_fn:
                 new_package_files.append(new_fn)
         if new_package_files:
-            for _module in _iter_rewritable_modules(new_package_files):
-                yield _module
+            yield from _iter_rewritable_modules(new_package_files)
 
 
-class Config(object):
-    """ access to configuration values, pluginmanager and plugin hooks.  """
+class Config:
+    """
+    Access to configuration values, pluginmanager and plugin hooks.
 
-    def __init__(self, pluginmanager):
-        #: access to command line option as attributes.
-        #: (deprecated), use :py:func:`getoption() <_pytest.config.Config.getoption>` instead
-        self.option = argparse.Namespace()
+    :ivar PytestPluginManager pluginmanager: the plugin manager handles plugin registration and hook invocation.
+
+    :ivar argparse.Namespace option: access to command line option as attributes.
+
+    :ivar InvocationParams invocation_params:
+
+        Object containing the parameters regarding the ``pytest.main``
+        invocation.
+
+        Contains the following read-only attributes:
+
+        * ``args``: tuple of command-line arguments as passed to ``pytest.main()``.
+        * ``plugins``: list of extra plugins, might be None.
+        * ``dir``: directory where ``pytest.main()`` was invoked from.
+    """
+
+    @attr.s(frozen=True)
+    class InvocationParams:
+        """Holds parameters passed during ``pytest.main()``
+
+        .. versionadded:: 5.1
+
+        .. note::
+
+            Note that the environment variable ``PYTEST_ADDOPTS`` and the ``addopts``
+            ini option are handled by pytest, not being included in the ``args`` attribute.
+
+            Plugins accessing ``InvocationParams`` must be aware of that.
+        """
+
+        args = attr.ib(converter=tuple)
+        plugins = attr.ib()
+        dir = attr.ib(type=Path)
+
+    def __init__(self, pluginmanager, *, invocation_params=None):
         from .argparsing import Parser, FILE_OR_DIR
 
+        if invocation_params is None:
+            invocation_params = self.InvocationParams(
+                args=(), plugins=None, dir=Path().resolve()
+            )
+
+        self.option = argparse.Namespace()
+        self.invocation_params = invocation_params
+
         _a = FILE_OR_DIR
         self._parser = Parser(
-            usage="%%(prog)s [options] [%s] [%s] [...]" % (_a, _a),
+            usage="%(prog)s [options] [{}] [{}] [...]".format(_a, _a),
             processopt=self._processopt,
         )
-        #: a pluginmanager instance
         self.pluginmanager = pluginmanager
         self.trace = self.pluginmanager.trace.root.get("config")
         self.hook = self.pluginmanager.hook
-        self._inicache = {}
-        self._override_ini = ()
-        self._opt2dest = {}
-        self._cleanup = []
+        self._inicache = {}  # type: Dict[str, Any]
+        self._override_ini = ()  # type: Sequence[str]
+        self._opt2dest = {}  # type: Dict[str, str]
+        self._cleanup = []  # type: List[Callable[[], None]]
         self.pluginmanager.register(self, "pytestconfig")
         self._configured = False
-        self.invocation_dir = py.path.local()
-        self.hook.pytest_addoption.call_historic(kwargs=dict(parser=self._parser))
+        self.hook.pytest_addoption.call_historic(
+            kwargs=dict(parser=self._parser, pluginmanager=self.pluginmanager)
+        )
+
+    @property
+    def invocation_dir(self):
+        """Backward compatibility"""
+        return py.path.local(str(self.invocation_params.dir))
 
     def add_cleanup(self, func):
         """ Add a function to be called when the config object gets out of
@@ -720,7 +773,9 @@ class Config(object):
     def _do_configure(self):
         assert not self._configured
         self._configured = True
-        self.hook.pytest_configure.call_historic(kwargs=dict(config=self))
+        with warnings.catch_warnings():
+            warnings.simplefilter("default")
+            self.hook.pytest_configure.call_historic(kwargs=dict(config=self))
 
     def _ensure_unconfigure(self):
         if self._configured:
@@ -781,7 +836,7 @@ class Config(object):
 
     @classmethod
     def fromdictargs(cls, option_dict, args):
-        """ constructor useable for subprocesses. """
+        """ constructor usable for subprocesses. """
         config = get_config(args)
         config.option.__dict__.update(option_dict)
         config.parse(args, addopts=False)
@@ -801,7 +856,7 @@ class Config(object):
     def pytest_load_initial_conftests(self, early_config):
         self.pluginmanager._set_initial_conftests(early_config.known_args_namespace)
 
-    def _initini(self, args):
+    def _initini(self, args) -> None:
         ns, unknown_args = self._parser.parse_known_and_unknown_args(
             args, namespace=copy.copy(self.option)
         )
@@ -902,8 +957,7 @@ class Config(object):
             self.hook.pytest_load_initial_conftests(
                 early_config=self, args=args, parser=self._parser
             )
-        except ConftestImportFailure:
-            e = sys.exc_info()[1]
+        except ConftestImportFailure as e:
             if ns.help or ns.version:
                 # we don't want to prevent --help/--version to work
                 # so just let is pass and print a warning at the end
@@ -940,7 +994,6 @@ class Config(object):
         assert not hasattr(
             self, "args"
         ), "can only parse cmdline args at most once per Config object"
-        self._origargs = args
         self.hook.pytest_addhooks.call_historic(
             kwargs=dict(pluginmanager=self.pluginmanager)
         )
@@ -969,10 +1022,10 @@ class Config(object):
         assert isinstance(x, list)
         x.append(line)  # modifies the cached list inline
 
-    def getini(self, name):
+    def getini(self, name: str):
         """ return configuration value from an :ref:`ini file <inifiles>`. If the
         specified name hasn't been registered through a prior
-        :py:func:`parser.addini <_pytest.config.Parser.addini>`
+        :py:func:`parser.addini <_pytest.config.argparsing.Parser.addini>`
         call (usually from a plugin), a ValueError is raised. """
         try:
             return self._inicache[name]
@@ -980,11 +1033,11 @@ class Config(object):
             self._inicache[name] = val = self._getini(name)
             return val
 
-    def _getini(self, name):
+    def _getini(self, name: str) -> Any:
         try:
             description, type, default = self._parser._inidict[name]
         except KeyError:
-            raise ValueError("unknown configuration value: %r" % (name,))
+            raise ValueError("unknown configuration value: {!r}".format(name))
         value = self._get_override_ini_value(name)
         if value is None:
             try:
@@ -1025,7 +1078,7 @@ class Config(object):
             values.append(relroot)
         return values
 
-    def _get_override_ini_value(self, name):
+    def _get_override_ini_value(self, name: str) -> Optional[str]:
         value = None
         # override_ini is a list of "ini=value" options
         # always use the last item if multiple values are set for same ini-name,
@@ -1040,7 +1093,7 @@ class Config(object):
                     value = user_ini_value
         return value
 
-    def getoption(self, name, default=notset, skip=False):
+    def getoption(self, name: str, default=notset, skip: bool = False):
         """ return command line option value.
 
         :arg name: name of the option.  You may also specify
@@ -1061,8 +1114,8 @@ class Config(object):
             if skip:
                 import pytest
 
-                pytest.skip("no %r option found" % (name,))
-            raise ValueError("no option named %r" % (name,))
+                pytest.skip("no {!r} option found".format(name))
+            raise ValueError("no option named {!r}".format(name))
 
     def getvalue(self, name, path=None):
         """ (deprecated, use getoption()) """
@@ -1150,4 +1203,4 @@ def _strtobool(val):
     elif val in ("n", "no", "f", "false", "off", "0"):
         return 0
     else:
-        raise ValueError("invalid truth value %r" % (val,))
+        raise ValueError("invalid truth value {!r}".format(val))
