@@ -17,7 +17,7 @@ if sys.gettrace():
 
 
 @pytest.hookimpl(hookwrapper=True, tryfirst=True)
-def pytest_collection_modifyitems(config, items):
+def pytest_collection_modifyitems(items):
     """Prefer faster tests.
 
     Use a hookwrapper to do this in the beginning, so e.g. --ff still works
