@@ -16,7 +16,7 @@ def test_resultlog_is_deprecated(testdir):
     result = testdir.runpytest("--result-log=%s" % testdir.tmpdir.join("result.log"))
     result.stdout.fnmatch_lines(
         [
-            "*--result-log is deprecated and scheduled for removal in pytest 6.0*",
+            "*--result-log is deprecated, please try the new pytest-reportlog plugin.",
             "*See https://docs.pytest.org/en/latest/deprecations.html#result-log-result-log for more information*",
         ]
     )
@@ -44,3 +44,32 @@ def test_external_plugins_integrated(testdir, plugin):
 
     with pytest.warns(pytest.PytestConfigWarning):
         testdir.parseconfig("-p", plugin)
+
+
+@pytest.mark.parametrize("junit_family", [None, "legacy", "xunit2"])
+def test_warn_about_imminent_junit_family_default_change(testdir, junit_family):
+    """Show a warning if junit_family is not defined and --junitxml is used (#6179)"""
+    testdir.makepyfile(
+        """
+        def test_foo():
+            pass
+    """
+    )
+    if junit_family:
+        testdir.makeini(
+            """
+            [pytest]
+            junit_family={junit_family}
+        """.format(
+                junit_family=junit_family
+            )
+        )
+
+    result = testdir.runpytest("--junit-xml=foo.xml")
+    warning_msg = (
+        "*PytestDeprecationWarning: The 'junit_family' default value will change*"
+    )
+    if junit_family:
+        result.stdout.no_fnmatch_line(warning_msg)
+    else:
+        result.stdout.fnmatch_lines([warning_msg])
