@@ -937,7 +937,7 @@ class TestFDCapture:
         cap.done()
         assert s == "hello\n"
 
-    def test_stdin(self, tmpfile):
+    def test_stdin(self):
         cap = capture.FDCapture(0)
         cap.start()
         x = os.read(0, 100).strip()
@@ -958,7 +958,7 @@ class TestFDCapture:
             stmp = stmp_file.read()
             assert stmp == data2
 
-    def test_simple_resume_suspend(self, tmpfile):
+    def test_simple_resume_suspend(self):
         with saved_fd(1):
             cap = capture.FDCapture(1)
             cap.start()
@@ -1367,6 +1367,8 @@ def test_crash_on_closing_tmpfile_py27(testdir):
             printing.wait()
     """
     )
+    # Do not consider plugins like hypothesis, which might output to stderr.
+    testdir.monkeypatch.setenv("PYTEST_DISABLE_PLUGIN_AUTOLOAD", "1")
     result = testdir.runpytest_subprocess(str(p))
     assert result.ret == 0
     assert result.stderr.str() == ""
