@@ -1,15 +1,8 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 import re
 import sys
 import textwrap
 
-import six
-
 import pytest
 from _pytest.monkeypatch import MonkeyPatch
 
@@ -24,7 +17,7 @@ def mp():
 
 
 def test_setattr():
-    class A(object):
+    class A:
         x = 1
 
     monkeypatch = MonkeyPatch()
@@ -47,7 +40,7 @@ def test_setattr():
     assert A.x == 5
 
 
-class TestSetattrWithImportPath(object):
+class TestSetattrWithImportPath:
     def test_string_expression(self, monkeypatch):
         monkeypatch.setattr("os.path.abspath", lambda x: "hello2")
         assert os.path.abspath("123") == "hello2"
@@ -89,7 +82,7 @@ class TestSetattrWithImportPath(object):
 
 
 def test_delattr():
-    class A(object):
+    class A:
         x = 1
 
     monkeypatch = MonkeyPatch()
@@ -200,30 +193,14 @@ def test_delenv():
             del os.environ[name]
 
 
-class TestEnvironWarnings(object):
+class TestEnvironWarnings:
     """
     os.environ keys and values should be native strings, otherwise it will cause problems with other modules (notably
     subprocess). On Python 2 os.environ accepts anything without complaining, while Python 3 does the right thing
     and raises an error.
     """
 
-    VAR_NAME = u"PYTEST_INTERNAL_MY_VAR"
-
-    @pytest.mark.skipif(not six.PY2, reason="Python 2 only test")
-    def test_setenv_unicode_key(self, monkeypatch):
-        with pytest.warns(
-            pytest.PytestWarning,
-            match="Environment variable name {!r} should be str".format(self.VAR_NAME),
-        ):
-            monkeypatch.setenv(self.VAR_NAME, "2")
-
-    @pytest.mark.skipif(not six.PY2, reason="Python 2 only test")
-    def test_delenv_unicode_key(self, monkeypatch):
-        with pytest.warns(
-            pytest.PytestWarning,
-            match="Environment variable name {!r} should be str".format(self.VAR_NAME),
-        ):
-            monkeypatch.delenv(self.VAR_NAME, raising=False)
+    VAR_NAME = "PYTEST_INTERNAL_MY_VAR"
 
     def test_setenv_non_str_warning(self, monkeypatch):
         value = 2
@@ -349,14 +326,12 @@ def test_importerror(testdir):
     result = testdir.runpytest()
     result.stdout.fnmatch_lines(
         """
-        *import error in package.a: No module named {0}doesnotexist{0}*
-    """.format(
-            "'" if sys.version_info > (3, 0) else ""
-        )
+        *import error in package.a: No module named 'doesnotexist'*
+    """
     )
 
 
-class SampleNew(object):
+class SampleNew:
     @staticmethod
     def hello():
         return True
@@ -366,7 +341,7 @@ class SampleNewInherit(SampleNew):
     pass
 
 
-class SampleOld(object):
+class SampleOld:
     # oldstyle on python2
     @staticmethod
     def hello():
@@ -393,7 +368,7 @@ def test_issue156_undo_staticmethod(Sample):
 
 
 def test_undo_class_descriptors_delattr():
-    class SampleParent(object):
+    class SampleParent:
         @classmethod
         def hello(_cls):
             pass
