@@ -82,6 +82,47 @@ class TestPaste:
     def pastebin(self, request):
         return request.config.pluginmanager.getplugin("pastebin")
 
+    @pytest.fixture
+    def mocked_urlopen_fail(self, monkeypatch):
+        """
+        monkeypatch the actual urlopen call to emulate a HTTP Error 400
+        """
+        calls = []
+
+        import urllib.error
+        import urllib.request
+
+        def mocked(url, data):
+            calls.append((url, data))
+            raise urllib.error.HTTPError(url, 400, "Bad request", None, None)
+
+        monkeypatch.setattr(urllib.request, "urlopen", mocked)
+        return calls
+
+    @pytest.fixture
+    def mocked_urlopen_invalid(self, monkeypatch):
+        """
+        monkeypatch the actual urlopen calls done by the internal plugin
+        function that connects to bpaste service, but return a url in an
+        unexpected format
+        """
+        calls = []
+
+        def mocked(url, data):
+            calls.append((url, data))
+
+            class DummyFile:
+                def read(self):
+                    # part of html of a normal response
+                    return b'View <a href="/invalid/3c0c6750bd">raw</a>.'
+
+            return DummyFile()
+
+        import urllib.request
+
+        monkeypatch.setattr(urllib.request, "urlopen", mocked)
+        return calls
+
     @pytest.fixture
     def mocked_urlopen(self, monkeypatch):
         """
@@ -105,13 +146,26 @@ class TestPaste:
         monkeypatch.setattr(urllib.request, "urlopen", mocked)
         return calls
 
+    def test_pastebin_invalid_url(self, pastebin, mocked_urlopen_invalid):
+        result = pastebin.create_new_paste(b"full-paste-contents")
+        assert (
+            result
+            == "bad response: invalid format ('View <a href=\"/invalid/3c0c6750bd\">raw</a>.')"
+        )
+        assert len(mocked_urlopen_invalid) == 1
+
+    def test_pastebin_http_error(self, pastebin, mocked_urlopen_fail):
+        result = pastebin.create_new_paste(b"full-paste-contents")
+        assert result == "bad response: HTTP Error 400: Bad request"
+        assert len(mocked_urlopen_fail) == 1
+
     def test_create_new_paste(self, pastebin, mocked_urlopen):
         result = pastebin.create_new_paste(b"full-paste-contents")
         assert result == "https://bpaste.net/show/3c0c6750bd"
         assert len(mocked_urlopen) == 1
         url, data = mocked_urlopen[0]
         assert type(data) is bytes
-        lexer = "python3"
+        lexer = "text"
         assert url == "https://bpaste.net"
         assert "lexer=%s" % lexer in data.decode()
         assert "code=full-paste-contents" in data.decode()
@@ -127,4 +181,4 @@ class TestPaste:
 
         monkeypatch.setattr(urllib.request, "urlopen", response)
         result = pastebin.create_new_paste(b"full-paste-contents")
-        assert result == "bad response: something bad occurred"
+        assert result == "bad response: invalid format ('something bad occurred')"
