@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+import os.path
 import sys
 
 import py
@@ -54,6 +54,10 @@ class TestPort:
     def test_matching(self, match, pattern, path):
         assert match(pattern, path)
 
+    def test_matching_abspath(self, match):
+        abspath = os.path.abspath(os.path.join("tests/foo.py"))
+        assert match("tests/foo.py", abspath)
+
     @pytest.mark.parametrize(
         "pattern, path",
         [
