@@ -115,7 +115,7 @@ class TestEvaluator:
         )
 
     def test_skipif_class(self, testdir):
-        item, = testdir.getitems(
+        (item,) = testdir.getitems(
             """
             import pytest
             class TestClass(object):
@@ -731,23 +731,37 @@ def test_skipif_class(testdir):
 def test_skipped_reasons_functional(testdir):
     testdir.makepyfile(
         test_one="""
+            import pytest
             from conftest import doskip
+
             def setup_function(func):
                 doskip()
+
             def test_func():
                 pass
+
             class TestClass(object):
                 def test_method(self):
                     doskip()
-       """,
+
+                @pytest.mark.skip("via_decorator")
+                def test_deco(self):
+                    assert 0
+        """,
         conftest="""
-            import pytest
+            import pytest, sys
             def doskip():
+                assert sys._getframe().f_lineno == 3
                 pytest.skip('test')
         """,
     )
     result = testdir.runpytest("-rs")
-    result.stdout.fnmatch_lines(["*SKIP*2*conftest.py:4: test"])
+    result.stdout.fnmatch_lines_random(
+        [
+            "SKIPPED [[]2[]] */conftest.py:4: test",
+            "SKIPPED [[]1[]] test_one.py:14: via_decorator",
+        ]
+    )
     assert result.ret == 0
 
 
@@ -886,7 +900,7 @@ def test_errors_in_xfail_skip_expressions(testdir):
             "    syntax error",
             markline,
             "SyntaxError: invalid syntax",
-            "*1 pass*2 error*",
+            "*1 pass*2 errors*",
         ]
     )
 
@@ -949,7 +963,7 @@ def test_xfail_test_setup_exception(testdir):
     result = testdir.runpytest(p)
     assert result.ret == 0
     assert "xfailed" in result.stdout.str()
-    assert "xpassed" not in result.stdout.str()
+    result.stdout.no_fnmatch_line("*xpassed*")
 
 
 def test_imperativeskip_on_xfail_test(testdir):
