@@ -3,6 +3,7 @@ terminal reporting of the full testing process.
 """
 import collections
 import os
+import re
 import sys
 import textwrap
 from io import StringIO
@@ -21,10 +22,15 @@ from _pytest.terminal import getreportopt
 from _pytest.terminal import TerminalReporter
 
 DistInfo = collections.namedtuple("DistInfo", ["project_name", "version"])
-RED = r"\x1b\[31m"
-GREEN = r"\x1b\[32m"
-YELLOW = r"\x1b\[33m"
-RESET = r"\x1b\[0m"
+
+COLORS = {
+    "red": "\x1b[31m",
+    "green": "\x1b[32m",
+    "yellow": "\x1b[33m",
+    "bold": "\x1b[1m",
+    "reset": "\x1b[0m",
+}
+RE_COLORS = {k: re.escape(v) for k, v in COLORS.items()}
 
 
 class Option:
@@ -621,7 +627,7 @@ class TestTerminalFunctional:
         if request.config.pluginmanager.list_plugin_distinfo():
             result.stdout.fnmatch_lines(["plugins: *"])
 
-    def test_header(self, testdir, request):
+    def test_header(self, testdir):
         testdir.tmpdir.join("tests").ensure_dir()
         testdir.tmpdir.join("gui").ensure_dir()
 
@@ -687,7 +693,7 @@ class TestTerminalFunctional:
         """
         )
 
-    def test_verbose_reporting(self, verbose_testfile, testdir, pytestconfig):
+    def test_verbose_reporting(self, verbose_testfile, testdir):
         result = testdir.runpytest(
             verbose_testfile, "-v", "-Walways::pytest.PytestWarning"
         )
@@ -790,8 +796,15 @@ def test_pass_reporting_on_fail(testdir):
 def test_pass_output_reporting(testdir):
     testdir.makepyfile(
         """
+        def setup_module():
+            print("setup_module")
+
+        def teardown_module():
+            print("teardown_module")
+
         def test_pass_has_output():
             print("Four score and seven years ago...")
+
         def test_pass_no_output():
             pass
     """
@@ -806,8 +819,12 @@ def test_pass_output_reporting(testdir):
         [
             "*= PASSES =*",
             "*_ test_pass_has_output _*",
+            "*- Captured stdout setup -*",
+            "setup_module",
             "*- Captured stdout call -*",
             "Four score and seven years ago...",
+            "*- Captured stdout teardown -*",
+            "teardown_module",
             "*= short test summary info =*",
             "PASSED test_pass_output_reporting.py::test_pass_has_output",
             "PASSED test_pass_output_reporting.py::test_pass_no_output",
@@ -817,10 +834,70 @@ def test_pass_output_reporting(testdir):
 
 
 def test_color_yes(testdir):
-    testdir.makepyfile("def test_this(): assert 1")
-    result = testdir.runpytest("--color=yes")
-    assert "test session starts" in result.stdout.str()
-    assert "\x1b[1m" in result.stdout.str()
+    p1 = testdir.makepyfile(
+        """
+        def fail():
+            assert 0
+
+        def test_this():
+            fail()
+        """
+    )
+    result = testdir.runpytest("--color=yes", str(p1))
+    if sys.version_info < (3, 6):
+        # py36 required for ordered markup
+        output = result.stdout.str()
+        assert "test session starts" in output
+        assert "\x1b[1m" in output
+        return
+    result.stdout.fnmatch_lines(
+        [
+            line.format(**COLORS).replace("[", "[[]")
+            for line in [
+                "{bold}=*= test session starts =*={reset}",
+                "collected 1 item",
+                "",
+                "test_color_yes.py {red}F{reset}{red} * [100%]{reset}",
+                "",
+                "=*= FAILURES =*=",
+                "{red}{bold}_*_ test_this _*_{reset}",
+                "",
+                "{bold}    def test_this():{reset}",
+                "{bold}>       fail(){reset}",
+                "",
+                "{bold}{red}test_color_yes.py{reset}:5: ",
+                "_ _ * _ _*",
+                "",
+                "{bold}    def fail():{reset}",
+                "{bold}>       assert 0{reset}",
+                "{bold}{red}E       assert 0{reset}",
+                "",
+                "{bold}{red}test_color_yes.py{reset}:2: AssertionError",
+                "{red}=*= {red}{bold}1 failed{reset}{red} in *s{reset}{red} =*={reset}",
+            ]
+        ]
+    )
+    result = testdir.runpytest("--color=yes", "--tb=short", str(p1))
+    result.stdout.fnmatch_lines(
+        [
+            line.format(**COLORS).replace("[", "[[]")
+            for line in [
+                "{bold}=*= test session starts =*={reset}",
+                "collected 1 item",
+                "",
+                "test_color_yes.py {red}F{reset}{red} * [100%]{reset}",
+                "",
+                "=*= FAILURES =*=",
+                "{red}{bold}_*_ test_this _*_{reset}",
+                "{bold}{red}test_color_yes.py{reset}:5: in test_this",
+                "{bold}    fail(){reset}",
+                "{bold}{red}test_color_yes.py{reset}:2: in fail",
+                "{bold}    assert 0{reset}",
+                "{bold}{red}E   assert 0{reset}",
+                "{red}=*= {red}{bold}1 failed{reset}{red} in *s{reset}{red} =*={reset}",
+            ]
+        ]
+    )
 
 
 def test_color_no(testdir):
@@ -932,7 +1009,7 @@ def test_tbstyle_short(testdir):
     assert "assert x" in s
 
 
-def test_traceconfig(testdir, monkeypatch):
+def test_traceconfig(testdir):
     result = testdir.runpytest("--traceconfig")
     result.stdout.fnmatch_lines(["*active plugins*"])
     assert result.ret == ExitCode.NO_TESTS_COLLECTED
@@ -1537,18 +1614,15 @@ class TestProgressOutputStyle:
                 def test_foobar(i): raise ValueError()
             """,
         )
-        output = testdir.runpytest()
-        output.stdout.re_match_lines(
+        result = testdir.runpytest()
+        result.stdout.re_match_lines(
             [
-                r"test_bar.py ({green}\.{reset}){{10}}{green} \s+ \[ 50%\]{reset}".format(
-                    green=GREEN, reset=RESET
-                ),
-                r"test_foo.py ({green}\.{reset}){{5}}{yellow} \s+ \[ 75%\]{reset}".format(
-                    green=GREEN, reset=RESET, yellow=YELLOW
-                ),
-                r"test_foobar.py ({red}F{reset}){{5}}{red} \s+ \[100%\]{reset}".format(
-                    reset=RESET, red=RED
-                ),
+                line.format(**RE_COLORS)
+                for line in [
+                    r"test_bar.py ({green}\.{reset}){{10}}{green} \s+ \[ 50%\]{reset}",
+                    r"test_foo.py ({green}\.{reset}){{5}}{yellow} \s+ \[ 75%\]{reset}",
+                    r"test_foobar.py ({red}F{reset}){{5}}{red} \s+ \[100%\]{reset}",
+                ]
             ]
         )
 
